// SegmentView.cpp : implementation of the CSegmentView class
//

#include "stdafx.h"
#include "Segment.h"

#include "ImageDoc.h"
#include "SegmentView.h"
#include "MonochromeBitmap.h"
#include "BlobStatsDlg.h"
#include "QuadTree.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSegmentView

IMPLEMENT_DYNCREATE(CSegmentView, CScrollView)

BEGIN_MESSAGE_MAP(CSegmentView, CScrollView)
	//{{AFX_MSG_MAP(CSegmentView)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_BLOB_STATISTICS, OnBlobStatistics)
	ON_UPDATE_COMMAND_UI(ID_BLOB_STATISTICS, OnUpdateBlobStatistics)
	ON_COMMAND(ID_BLOB_CHOOSETEMPLATE, OnBlobChoosetemplate)
	ON_UPDATE_COMMAND_UI(ID_BLOB_CHOOSETEMPLATE, OnUpdateBlobChoosetemplate)
	ON_COMMAND(ID_BLOB_MATCH, OnBlobMatch)
	ON_UPDATE_COMMAND_UI(ID_BLOB_MATCH, OnUpdateBlobMatch)
	ON_WM_CREATE()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_IMAGE_FINDBULLSEYES, OnImageFindbullseyes)
	ON_UPDATE_COMMAND_UI(ID_IMAGE_FINDBULLSEYES, OnUpdateImageFindbullseyes)
	ON_COMMAND(ID_VIEW_REVERSEVIDEO, OnViewReversevideo)
	ON_UPDATE_COMMAND_UI(ID_VIEW_REVERSEVIDEO, OnUpdateViewReversevideo)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSegmentView construction/destruction

CSegmentView::CSegmentView()
{
	m_pSelectedBlob = NULL;
	m_pTemplateBlob = NULL;
	m_bReverseVideo = FALSE;
}

CSegmentView::~CSegmentView()
{
}

BOOL CSegmentView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSegmentView drawing

//-----------------------------------------------------------------
//
// CSegmentView::OnDraw - Draw the bitmap and a host of blobs:
//		currently selected blob
//		template matching blob
//		bullseye blobs
//
//-----------------------------------------------------------------

void CSegmentView::OnDraw(CDC* pDC)
{
	CImageDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect crView;
	GetScrolledViewRect(crView);
	DWORD dwRop = m_bReverseVideo?NOTSRCCOPY:SRCCOPY;
	pDoc->StretchBlt(*pDC,crView,crView,dwRop);
	if (! pDoc->IsEmpty()) {
		DrawBlobs(pDC);
	}
}

//-----------------------------------------------------------------
//
// CSegmentView::DrawBlobs - Oh, yeah, here's the routine in
//							 which we enumerate the blobs to draw
//
//-----------------------------------------------------------------

void CSegmentView::DrawBlobs(CDC *pDC)
{
	CImageDoc* pDoc = GetDocument();

	CMonochromeBitmap &mb = pDoc->GetMonochromeBitmap();
	int nBlobs = mb.GetBlobCount();
	RGBQUAD color;
	CRect crView;
	GetScrolledViewRect(crView);
	vector<CBlob *> vBlobs;
	CQuadTree<CBlob> &tree = mb.GetQuadTree();
	tree.GetItems(crView,vBlobs);

	for (vector<CBlob *>::iterator ppBlob = vBlobs.begin(); ppBlob != vBlobs.end(); ppBlob++) {
		CBlob *pBlob = *ppBlob;
		CRect crBlob;
		pBlob->GetRect(crBlob);
		memset(&color,0,sizeof(color));
		if (pBlob == m_pSelectedBlob) {
			color.rgbGreen = 255;
		} else {
			color.rgbRed = 255;
		}
		PaintBlob(pDC,*pBlob,crBlob.TopLeft(),color);
	}

	if (m_pTemplateBlob != NULL && m_pSelectedBlob != NULL) {
		CRect crTemplate;
		GetTemplateRect(crTemplate);
		memset(&color,0,sizeof(color));
		color.rgbBlue = 255;

		PaintBlob(pDC,*m_pTemplateBlob,crTemplate.TopLeft(),color);
	}
	
	memset(&color,0,sizeof(color));
	color.rgbGreen = 255;
	color.rgbRed = 255;
	for (vector<CBlob *>::iterator ipBlob = m_blobBullseye.begin(); ipBlob != m_blobBullseye.end(); ipBlob++) {
		CRect crBlob;
		(*ipBlob)->GetRect(crBlob);
		PaintBlob(pDC,**ipBlob,crBlob.TopLeft(),color);
	}
}

//-----------------------------------------------------------------
//
// CSegmentView::PaintBlob - This paints one blob in the dc.
//
// pDC - paint the blob outline on this DC
// blob - paint this blob's outline.
// ptOffset - offset the bitmap by this much.
// color - paint the outline using this color.
//
//-----------------------------------------------------------------

void CSegmentView::PaintBlob(CDC * pDC, CBlob & blob, CPoint ptOffset,RGBQUAD &color)
{
	CRect crView;
	GetScrolledViewRect(crView);
	CRect crScratch;
	blob.GetRect(crScratch);
	CRect crBlob(ptOffset,crScratch.Size());
	CRect crIntersect;

	if (crIntersect.IntersectRect(crBlob,crView)) {
		// The blob is at least partially within the view.

		CString cszBuffer;
		BITMAPINFO *pBMI = (BITMAPINFO *)(cszBuffer.GetBuffer(sizeof(BITMAPINFO) + sizeof(RGBQUAD)));
		RGBQUAD *pColors = pBMI->bmiColors;
		CByteArray bMemory;
		bMemory.SetSize(blob.GetDIBSize());
		BYTE *pMemory = bMemory.GetData();
		blob.GetDIB(pBMI->bmiHeader,pMemory);

		// Why does Microsoft think so little of StretchDIBits
		// that it doesn't provide a wrapper for it in MFC?

		// We do that color masking thing here.
		// First, AND the inverse of the monochrome bitmap onto
		// the DC, then OR the real thing.

		memset(pColors,0,sizeof(RGBQUAD)*2);
		pColors[0].rgbRed = pColors[0].rgbBlue = pColors[0].rgbGreen = 255;

		StretchDIBits(pDC->m_hDC,crBlob.left,crBlob.top,
					  crBlob.Width(),crBlob.Height(),
					  0,0,
					  crBlob.Width(),crBlob.Height(),
					  pMemory,pBMI,
					  DIB_RGB_COLORS,SRCAND);
		pColors[0].rgbRed = pColors[0].rgbBlue = pColors[0].rgbGreen = 0;
		memcpy(pColors + 1,&color,sizeof(color));
		StretchDIBits(pDC->m_hDC,crBlob.left,crBlob.top,
					  crBlob.Width(),crBlob.Height(),
					  0,0,
					  crBlob.Width(),crBlob.Height(),
					  pMemory,pBMI,
					  DIB_RGB_COLORS,SRCPAINT);
	}
}

//-----------------------------------------------------------------
//
// CSegmentView::OnInitialUpdate - Figure out the image size on the
//						first update. Also, build blobs for the image.
//
//-----------------------------------------------------------------

void CSegmentView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	CImageDoc *pDoc = GetDocument();
	CSize sizeTotal(pDoc->Width(),pDoc->Height());
	// TODO: calculate the total size of this view
	SetScrollSizes(MM_TEXT, sizeTotal);
	if (sizeTotal.cx != 0 && sizeTotal.cy != 0) {
		pDoc->GetMonochromeBitmap().BuildBlobList();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSegmentView printing

//-----------------------------------------------------------------
//
// CSegmentView::OnPreparePrinting - I have no idea if printing really works...
//
//-----------------------------------------------------------------

BOOL CSegmentView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSegmentView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CSegmentView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CSegmentView diagnostics

#ifdef _DEBUG
void CSegmentView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CSegmentView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CImageDoc* CSegmentView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CImageDoc)));
	return (CImageDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSegmentView message handlers

//-----------------------------------------------------------------
//
// CSegmentView::GetScrolledViewRect - This returns the rectangle
//				being viewed by the user.
//
//-----------------------------------------------------------------

void CSegmentView::GetScrolledViewRect(CRect & cr)
{
	CRect crView;
	GetClientRect(&crView);
	CPoint ptScroll(GetScrollPos(SB_HORZ),GetScrollPos(SB_VERT));
	cr = CRect(ptScroll,crView.Size());
}

//-----------------------------------------------------------------
//
// CSegmentView::OnLButtonDown - Select a blob on left mouse button down.
//					Also, match the selected blob against the template
//					blob and display the result in a fake tool tip.
//
//-----------------------------------------------------------------

void CSegmentView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CMonochromeBitmap &mb = GetDocument()->GetMonochromeBitmap();
	int nBlobs = mb.GetBlobCount();
	CPoint ptClient = point;
	ClientToBitmap(point);
	for (int i = 0; i < nBlobs; i++) {
		CBlob *pBlob = mb.GetBlob(i);
		if (pBlob->PtInBlob(point)) {
			// Invalidate the currently selected blob
			// and the blob-to-be.

			CRect cr;
			if (m_pSelectedBlob != NULL) {
				m_pSelectedBlob->GetRect(cr);
				BitmapToClient(cr);
				InvalidateRect(cr);
				if (m_pTemplateBlob) {
					GetTemplateRect(cr);
					BitmapToClient(cr);
					InvalidateRect(cr);
				}
			}

			m_pSelectedBlob = pBlob;
			pBlob->GetRect(cr);
			BitmapToClient(cr);
			InvalidateRect(cr);

			if (m_pTemplateBlob) {
				// Remember to paint the new template location too

				GetTemplateRect(cr);
				BitmapToClient(cr);
				InvalidateRect(cr);

				// Let's display the match inside the fake tool tip.

				double dfMatch = pBlob->MatchBlobs(*m_pTemplateBlob);
				CString csz;
				char buffer[20];
				AfxFormatString1(csz,IDS_SHORT_MATCH,gcvt(dfMatch,3,buffer));
				m_staticInfo.SetWindowText(csz);
				CClientDC dc(&m_staticInfo);
				CSize csClient = dc.GetTextExtent(csz);
				CRect crInfo(ptClient,csClient);
				AdjustWindowRect(&crInfo,WS_BORDER,FALSE);
				m_staticInfo.SetWindowPos(&wndTop,crInfo.left,crInfo.top,crInfo.Width(),crInfo.Height(),SWP_SHOWWINDOW);
			}
			break;
		}
	}
	
	CScrollView::OnLButtonDown(nFlags, point);
}

//-----------------------------------------------------------------
//
// CSegmentView::ClientToBitmap - Transform coordinates from client
//					relative to bitmap relative.
//
//-----------------------------------------------------------------

void CSegmentView::ClientToBitmap(CPoint &pt)
{
	pt.x += GetScrollPos(SB_HORZ);
	pt.y += GetScrollPos(SB_VERT);
}

void CSegmentView::BitmapToClient(CPoint &pt)
{
	pt.x -= GetScrollPos(SB_HORZ);
	pt.y -= GetScrollPos(SB_VERT);

}

void CSegmentView::ClientToBitmap(CRect & cr)
{
	ClientToBitmap(cr.TopLeft());
	ClientToBitmap(cr.BottomRight());
}

void CSegmentView::BitmapToClient(CRect & cr)
{
	BitmapToClient(cr.TopLeft());
	BitmapToClient(cr.BottomRight());
}

//-----------------------------------------------------------------
//
// CSegmentView::OnUpdate - It's safest to remove selection after
//						the image has changed.
//
//-----------------------------------------------------------------

void CSegmentView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	m_pSelectedBlob = NULL;	
	m_pTemplateBlob = NULL;
	m_blobBullseye.resize(0);
	CScrollView::OnUpdate(pSender,lHint,pHint);
}

//-----------------------------------------------------------------
//
// CSegmentView::OnBlobStatistics - Display statistics for the
//							currently selected blob.
//
//-----------------------------------------------------------------

void CSegmentView::OnBlobStatistics() 
{
	CBlobStatsDlg dlg(this);
	
	ASSERT(m_pSelectedBlob);
	dlg.m_nArea = m_pSelectedBlob->GetArea();
	CPoint ptCentroid = m_pSelectedBlob->GetCentroid();
	dlg.m_nCentroidX = ptCentroid.x;
	dlg.m_nCentroidY = ptCentroid.y;
	CRect cr;
	m_pSelectedBlob->GetRect(cr);
	dlg.m_nTop = cr.top;
	dlg.m_nLeft = cr.left;
	dlg.m_nBottom = cr.bottom;
	dlg.m_nRight = cr.right;
	dlg.m_nLoops = m_pSelectedBlob->m_nLoops;
	dlg.DoModal();
}

void CSegmentView::OnUpdateBlobStatistics(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pSelectedBlob != NULL);	
}

//-----------------------------------------------------------------
//
// CSegmentView::OnBlobChoosetemplate - Make the currently selected
//									blob into the template blob.
//
//-----------------------------------------------------------------

void CSegmentView::OnBlobChoosetemplate() 
{
	// We have to invalidate the old template, or else
	// the old template's junk never clears.

	CRect cr;
	if (m_pTemplateBlob) {
		GetTemplateRect(cr);
		BitmapToClient(cr);
		InvalidateRect(cr);
	}

	m_pTemplateBlob = m_pSelectedBlob;

	// Might as well paint where the new one goes...
	GetTemplateRect(cr);
	BitmapToClient(cr);
	InvalidateRect(cr);
}

void CSegmentView::OnUpdateBlobChoosetemplate(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pSelectedBlob != NULL);
	
}

//-----------------------------------------------------------------
//
// CSegmentView::OnBlobMatch - Match the currently selected blob
//								against the template blob
//
//-----------------------------------------------------------------

void CSegmentView::OnBlobMatch() 
{
	CMonochromeBitmap &mb = GetDocument()->GetMonochromeBitmap();
	ASSERT(m_pSelectedBlob);
	ASSERT(m_pTemplateBlob);

	double dfMatch = m_pSelectedBlob->MatchBlobs(*m_pTemplateBlob);

	char buffer[20];
	gcvt(dfMatch,3,buffer);
	CString csz;
	AfxFormatString1(csz,IDS_MATCH1,buffer);
	MessageBox(csz);
}

void CSegmentView::OnUpdateBlobMatch(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pSelectedBlob && m_pTemplateBlob);
}

//-----------------------------------------------------------------
//
// CSegmentView::OnCreate - Create the fake tool tip.
//
//-----------------------------------------------------------------

int CSegmentView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CScrollView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create our little fake tool tip.

	CRect crInfo(0,0,100,100);
	if (! m_staticInfo.Create(NULL,WS_CHILD|WS_BORDER,crInfo,this)) {
		return -1;
	}
	
	return 0;
}

//-----------------------------------------------------------------
//
// CSegmentView::OnLButtonUp - Kill the template matching tooltip on button up.
//
//-----------------------------------------------------------------

void CSegmentView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	m_staticInfo.ShowWindow(SW_HIDE);	
	CScrollView::OnLButtonUp(nFlags, point);
}

//-----------------------------------------------------------------
//
// CSegmentView::GetTemplateRect - We overlay the template blob outline
//					over the selected blob outline. This involves
//					matching their centroids (because that's how
//					the matching is done) and computing the offset
//					to the template blob's top left corner.
//
// cr - return template rectangle here.
//
//-----------------------------------------------------------------

void CSegmentView::GetTemplateRect(CRect & cr)
{
	ASSERT(m_pTemplateBlob);
	ASSERT(m_pSelectedBlob);

	CMonochromeBitmap &mb = GetDocument()->GetMonochromeBitmap();
	CPoint ptTemplate = m_pTemplateBlob->GetCentroid();
	CPoint ptSelection = m_pSelectedBlob->GetCentroid();

	CRect crTemplate;
	m_pTemplateBlob->GetRect(crTemplate);

	// Figure how far away the template center is from the top left corners.

	CSize csOffsetTemplate = ptTemplate - crTemplate.TopLeft();

	// The template rectangle starts at the selection center
	// minus the template offset.

	cr.TopLeft() = ptSelection - csOffsetTemplate;
	cr.BottomRight() = cr.TopLeft() + crTemplate.Size();
}

//-----------------------------------------------------------------
//
// CSegmentView::OnImageFindbullseyes - This builds a list of the
//						blobs in any "bullseyes" in the image. We
//						define a bullseye as a center with two or more
//						objects (that have at least one loop) whose
//						centroids are close.
//
//-----------------------------------------------------------------

void CSegmentView::OnImageFindbullseyes() 
{
	// We define a bullseye as three concentric objects (they can be
	// ellipses if the target is tilted) whose centroids fall within
	// the same 10 x 10 rectangle.

	CMonochromeBitmap &mb = GetDocument()->GetMonochromeBitmap();
	CQuadTree<CBlob> &qt = mb.GetQuadTree();
	
	// OK, this is simplicity and it runs in N log N rather than N * N.

	const int nBlobs = mb.GetBlobCount();
	for (int i = 0; i < nBlobs; i++) {
		CBlob *pBlob = mb.GetBlob(i);
		CRect cr(pBlob->GetCentroid(),CSize(0,0));

		// OK, if something is smaller than 5x5, then this rectangle
		// could possibly enclose another small point. If the inside
		// bullseye is really small, we still won't fail because the
		// larger one will pass the test.
		// Let's also throw in the contingency that the test object
		// has at least one loop. That excludes the center anyway.

		if (pBlob->GetArea() > 25 && pBlob->m_nLoops > 0) {
			cr.InflateRect(5,5);
			vector<CBlob *> vBlob;
			qt.GetItems(cr,vBlob);
			if (vBlob.size() >= 3) {
				m_blobBullseye.insert(m_blobBullseye.end(),vBlob.begin(),vBlob.end());
			}
		}
	}
	Invalidate();
}

void CSegmentView::OnUpdateImageFindbullseyes(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(GetDocument()->IsEmpty() == FALSE);
}

//-----------------------------------------------------------------
//
// CSegmentView::OnViewReversevideo - Flip the reverse video flag.
//
//-----------------------------------------------------------------

void CSegmentView::OnViewReversevideo() 
{
	m_bReverseVideo = ! m_bReverseVideo;
	Invalidate();
}

void CSegmentView::OnUpdateViewReversevideo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(GetDocument()->IsEmpty() == FALSE);
	pCmdUI->SetCheck(m_bReverseVideo);
	
}
